@@ -1,5 +1,5 @@
 /* server.c --- SASL CRAM-MD5 server side functions.
- * Copyright (C) 2009-2019 Simon Josefsson
+ * Copyright (C) 2009-2020 Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -44,6 +44,7 @@
 #include "printer.h"
 #include "gc.h"
 #include "memxor.h"
+#include "mechtools.h"
 
 #define DEFAULT_SALT_BYTES 12
 #define SNONCE_ENTROPY_BYTES 18
@@ -51,6 +52,7 @@
 struct scram_server_state
 {
   int plus;
+  int hash; // 0 is SHA1, 1 is SHA256
   int step;
   char *cbind;
   char *gs2header;		/* copy of client first gs2-header */
@@ -70,7 +72,7 @@ struct scram_server_state
 };
 
 static int
-scram_start (Gsasl_session * sctx, void **mech_data, int plus)
+scram_start (Gsasl_session * sctx, void **mech_data, int plus, int hash)
 {
   struct scram_server_state *state;
   char buf[MAX (SNONCE_ENTROPY_BYTES, DEFAULT_SALT_BYTES)];
@@ -82,12 +84,19 @@ scram_start (Gsasl_session * sctx, void **mech_data, int plus)
     return GSASL_MALLOC_ERROR;
 
   state->plus = plus;
+  state->hash = hash;
 
   rc = gsasl_nonce (buf, SNONCE_ENTROPY_BYTES);
   if (rc != GSASL_OK)
     goto end;
 
   rc = gsasl_base64_to (buf, SNONCE_ENTROPY_BYTES, &state->snonce, NULL);
+#if SCRAMDEBUG
+  if (hash == 0)
+    state->snonce = strdup ("3rfcNHYJY1ZVvWVs7j"); // SCRAM-SHA1
+  else
+    state->snonce = strdup ("%hvYDpWUa2RaTCAfuxFIlj)hNlF$k0"); // SCRAM-SHA256
+#endif
   if (rc != GSASL_OK)
     goto end;
 
@@ -124,24 +133,40 @@ end:
   return rc;
 }
 
+#ifdef USE_SCRAM_SHA1
 int
 _gsasl_scram_sha1_server_start (Gsasl_session * sctx, void **mech_data)
 {
-  return scram_start (sctx, mech_data, 0);
+  return scram_start (sctx, mech_data, 0, 0);
 }
 
 int
 _gsasl_scram_sha1_plus_server_start (Gsasl_session * sctx, void **mech_data)
 {
-  return scram_start (sctx, mech_data, 1);
+  return scram_start (sctx, mech_data, 1, 0);
+}
+#endif
+
+#ifdef USE_SCRAM_SHA256
+int
+_gsasl_scram_sha256_server_start (Gsasl_session * sctx, void **mech_data)
+{
+  return scram_start (sctx, mech_data, 0, 1);
 }
 
 int
-_gsasl_scram_sha1_server_step (Gsasl_session * sctx,
-			       void *mech_data,
-			       const char *input,
-			       size_t input_len,
-			       char **output, size_t * output_len)
+_gsasl_scram_sha256_plus_server_start (Gsasl_session * sctx, void **mech_data)
+{
+  return scram_start (sctx, mech_data, 1, 1);
+}
+#endif
+
+static int
+scram_server_step (Gsasl_session * sctx,
+		   void *mech_data,
+		   const char *input,
+		   size_t input_len,
+		   char **output, size_t * output_len)
 {
   struct scram_server_state *state = mech_data;
   int res = GSASL_MECHANISM_CALLED_TOO_MANY_TIMES;
@@ -209,15 +234,23 @@ _gsasl_scram_sha1_server_step (Gsasl_session * sctx,
 	/* Create new nonce. */
 	{
 	  size_t cnlen = strlen (state->cf.client_nonce);
+	  size_t snlen = strlen (state->snonce);
 
-	  state->sf.nonce = malloc (cnlen + SNONCE_ENTROPY_BYTES + 1);
+#if SCRAMDEBUG
+	  printf ("Server.cnonce: %s\n", state->cf.client_nonce);
+	  printf ("Server.snonce: %s\n", state->snonce);
+#endif
+
+	  state->sf.nonce = malloc (cnlen + snlen + 1);
 	  if (!state->sf.nonce)
 	    return GSASL_MALLOC_ERROR;
 
 	  memcpy (state->sf.nonce, state->cf.client_nonce, cnlen);
-	  memcpy (state->sf.nonce + cnlen, state->snonce,
-		  SNONCE_ENTROPY_BYTES);
-	  state->sf.nonce[cnlen + SNONCE_ENTROPY_BYTES] = '\0';
+	  memcpy (state->sf.nonce + cnlen, state->snonce, snlen);
+	  state->sf.nonce[cnlen + snlen] = '\0';
+#if SCRAMDEBUG
+	  printf ("Server.nonce: %s\n", state->sf.nonce);
+#endif
 	}
 
 	gsasl_property_set (sctx, GSASL_AUTHID, state->cf.username);
@@ -299,7 +332,7 @@ _gsasl_scram_sha1_server_step (Gsasl_session * sctx,
 	}
 
 	/* Base64 decode client proof and check that length matches
-	   SHA-1 size. */
+	   SHA1/SHA256 size. */
 	{
 	  size_t len;
 
@@ -307,10 +340,9 @@ _gsasl_scram_sha1_server_step (Gsasl_session * sctx,
 				  &state->clientproof, &len);
 	  if (rc != 0)
 	    return rc;
-	  if (len != 20)
+	  if ((state->hash == 0 && len != 20) || (state->hash == 1 && len != 32))
 	    return GSASL_MECHANISM_PARSE_ERROR;
 	}
-
 	{
 	  const char *p;
 
@@ -320,7 +352,7 @@ _gsasl_scram_sha1_server_step (Gsasl_session * sctx,
 	      Gc_rc err;
 	      char *salt;
 	      size_t saltlen;
-	      char saltedpassword[20];
+	      char saltedpassword[32];
 	      char *clientkey;
 	      char *preppasswd;
 
@@ -337,35 +369,92 @@ _gsasl_scram_sha1_server_step (Gsasl_session * sctx,
 		}
 
 	      /* SaltedPassword := Hi(password, salt) */
-	      err = gc_pbkdf2_sha1 (preppasswd, strlen (preppasswd),
-				    salt, saltlen,
-				    state->sf.iter, saltedpassword, 20);
+	      if (state->hash == 0)
+		err = gc_pbkdf2_sha1 (preppasswd, strlen (preppasswd),
+				      salt, saltlen,
+				      state->sf.iter, saltedpassword, 20);
+	      else
+		err = gc_pbkdf2_sha256 (preppasswd, strlen (preppasswd),
+				      salt, saltlen,
+				      state->sf.iter, saltedpassword, 32);
 	      gsasl_free (preppasswd);
 	      gsasl_free (salt);
 	      if (err != GC_OK)
 		return GSASL_MALLOC_ERROR;
+#if SCRAMDEBUG
+	      {
+		int i;
+		printf ("Server.SaltedPassword: ");
+		for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
+		  printf ("%02x", saltedpassword[i] & 0xFF);
+		printf ("\n");
+	      }
+#endif
 
 	      /* ClientKey := HMAC(SaltedPassword, "Client Key") */
 #define CLIENT_KEY "Client Key"
-	      rc = gsasl_hmac_sha1 (saltedpassword, 20,
-				    CLIENT_KEY, strlen (CLIENT_KEY),
-				    &clientkey);
+	      if (state->hash == 0)
+		rc = gsasl_hmac_sha1 (saltedpassword, 20,
+				      CLIENT_KEY, strlen (CLIENT_KEY),
+				      &clientkey);
+	      else
+		rc = _gsasl_hmac_sha256 (saltedpassword, 32,
+					 CLIENT_KEY, strlen (CLIENT_KEY),
+					 &clientkey);
 	      if (rc != 0)
 		return rc;
 
+#if SCRAMDEBUG
+	      {
+		int i;
+		printf ("Server.ClientKey: ");
+		for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
+		  printf ("%02x", clientkey[i] & 0xFF);
+		printf ("\n");
+	      }
+#endif
+
 	      /* StoredKey := H(ClientKey) */
-	      rc = gsasl_sha1 (clientkey, 20, &state->storedkey);
+	      if (state->hash == 0)
+		rc = gsasl_sha1 (clientkey, 20, &state->storedkey);
+	      else
+		rc = _gsasl_sha256 (clientkey, 32, &state->storedkey);
 	      free (clientkey);
 	      if (rc != 0)
 		return rc;
 
+#if SCRAMDEBUG
+	      {
+		int i;
+		printf ("Server.StoredKey: ");
+		for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
+		  printf ("%02x", state->storedkey[i] & 0xFF);
+		printf ("\n");
+	      }
+#endif
+
 	      /* ServerKey := HMAC(SaltedPassword, "Server Key") */
 #define SERVER_KEY "Server Key"
-	      rc = gsasl_hmac_sha1 (saltedpassword, 20,
-				    SERVER_KEY, strlen (SERVER_KEY),
-				    &state->serverkey);
+	      if (state->hash == 0)
+		rc = gsasl_hmac_sha1 (saltedpassword, 20,
+				      SERVER_KEY, strlen (SERVER_KEY),
+				      &state->serverkey);
+	      else
+		rc = _gsasl_hmac_sha256 (saltedpassword, 32,
+					 SERVER_KEY, strlen (SERVER_KEY),
+					 &state->serverkey);
 	      if (rc != 0)
 		return rc;
+
+#if SCRAMDEBUG
+	      {
+		int i;
+		printf ("Server.ServerKey: ");
+		for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
+		  printf ("%02x", state->serverkey[i] & 0xFF);
+		printf ("\n");
+	      }
+#endif
 	    }
 	  else
 	    return GSASL_NO_PASSWORD;
@@ -388,6 +477,9 @@ _gsasl_scram_sha1_server_step (Gsasl_session * sctx,
 	    if (n <= 0 || !state->authmessage)
 	      return GSASL_MALLOC_ERROR;
 	  }
+#if SCRAMDEBUG
+	  printf ("Server.AuthMessage: %s\n", state->authmessage);
+#endif
 
 	  /* Check client proof. */
 	  {
@@ -395,22 +487,67 @@ _gsasl_scram_sha1_server_step (Gsasl_session * sctx,
 	    char *maybe_storedkey;
 
 	    /* ClientSignature := HMAC(StoredKey, AuthMessage) */
-	    rc = gsasl_hmac_sha1 (state->storedkey, 20,
-				  state->authmessage,
-				  strlen (state->authmessage),
-				  &clientsignature);
+	    if (state->hash == 0)
+	      rc = gsasl_hmac_sha1 (state->storedkey, 20,
+				    state->authmessage,
+				    strlen (state->authmessage),
+				    &clientsignature);
+	    else
+	      rc = _gsasl_hmac_sha256 (state->storedkey, 32,
+				       state->authmessage,
+				       strlen (state->authmessage),
+				       &clientsignature);
 	    if (rc != 0)
 	      return rc;
 
-	    /* ClientKey := ClientProof XOR ClientSignature */
-	    memxor (clientsignature, state->clientproof, 20);
+#if SCRAMDEBUG
+	    {
+	      int i;
+	      printf ("Server.ClientSignature: ");
+	      for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
+		printf ("%02x", clientsignature[i] & 0xFF);
+	      printf ("\n");
+	    }
+#endif
 
-	    rc = gsasl_sha1 (clientsignature, 20, &maybe_storedkey);
+	    /* ClientKey := ClientProof XOR ClientSignature */
+	    if (state->hash == 0)
+	      memxor (clientsignature, state->clientproof, 20);
+	    else
+	      memxor (clientsignature, state->clientproof, 32);
+
+#if SCRAMDEBUG
+	    {
+	      int i;
+	      printf ("Server.ClientKey: ");
+	      for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
+		printf ("%02x", clientsignature[i] & 0xFF);
+	      printf ("\n");
+	    }
+#endif
+
+	    if (state->hash == 0)
+	      rc = gsasl_sha1 (clientsignature, 20, &maybe_storedkey);
+	    else
+	      rc = _gsasl_sha256 (clientsignature, 32, &maybe_storedkey);
 	    free (clientsignature);
 	    if (rc != 0)
 	      return rc;
 
-	    rc = memcmp (state->storedkey, maybe_storedkey, 20);
+#if SCRAMDEBUG
+	    {
+	      int i;
+	      printf ("Server.StoredKey: ");
+	      for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
+		printf ("%02x", maybe_storedkey[i] & 0xFF);
+	      printf ("\n");
+	    }
+#endif
+
+	    if (state->hash == 0)
+	      rc = memcmp (state->storedkey, maybe_storedkey, 20);
+	    else
+	      rc = memcmp (state->storedkey, maybe_storedkey, 32);
 	    free (maybe_storedkey);
 	    if (rc != 0)
 	      return GSASL_AUTHENTICATION_ERROR;
@@ -421,15 +558,35 @@ _gsasl_scram_sha1_server_step (Gsasl_session * sctx,
 	    char *serversignature;
 
 	    /* ServerSignature := HMAC(ServerKey, AuthMessage) */
-	    rc = gsasl_hmac_sha1 (state->serverkey, 20,
+	    if (state->hash == 0)
+	      rc = gsasl_hmac_sha1 (state->serverkey, 20,
 				  state->authmessage,
 				  strlen (state->authmessage),
 				  &serversignature);
+	    else
+	      rc = _gsasl_hmac_sha256 (state->serverkey, 32,
+				       state->authmessage,
+				       strlen (state->authmessage),
+				       &serversignature);
 	    if (rc != 0)
 	      return rc;
 
-	    rc = gsasl_base64_to (serversignature, 20,
+#if SCRAMDEBUG
+	    {
+	      int i;
+	      printf ("Server.ServerSignature: ");
+	      for (i = 0; i < (state->hash == 0 ? 20 : 32); i++)
+		printf ("%02x", serversignature[i] & 0xFF);
+	      printf ("\n");
+	    }
+#endif
+
+	    if (state->hash == 0)
+	      rc = gsasl_base64_to (serversignature, 20,
 				  &state->sl.verifier, NULL);
+	    else
+	      rc = gsasl_base64_to (serversignature, 32,
+				    &state->sl.verifier, NULL);
 	    free (serversignature);
 	    if (rc != 0)
 	      return rc;
@@ -453,8 +610,32 @@ _gsasl_scram_sha1_server_step (Gsasl_session * sctx,
   return res;
 }
 
-void
-_gsasl_scram_sha1_server_finish (Gsasl_session * sctx, void *mech_data)
+#ifdef USE_SCRAM_SHA1
+int
+_gsasl_scram_sha1_server_step (Gsasl_session * sctx,
+			       void *mech_data,
+			       const char *input,
+			       size_t input_len,
+			       char **output, size_t * output_len)
+{
+  return scram_server_step (sctx, mech_data, input, input_len, output, output_len);
+}
+#endif
+
+#ifdef USE_SCRAM_SHA256
+int
+_gsasl_scram_sha256_server_step (Gsasl_session * sctx,
+				 void *mech_data,
+				 const char *input,
+				 size_t input_len,
+				 char **output, size_t * output_len)
+{
+  return scram_server_step (sctx, mech_data, input, input_len, output, output_len);
+}
+#endif
+
+static void
+scram_server_finish (Gsasl_session * sctx, void *mech_data)
 {
   struct scram_server_state *state = mech_data;
 
@@ -478,3 +659,19 @@ _gsasl_scram_sha1_server_finish (Gsasl_session * sctx, void *mech_data)
 
   free (state);
 }
+
+#ifdef USE_SCRAM_SHA1
+void
+_gsasl_scram_sha1_server_finish (Gsasl_session * sctx, void *mech_data)
+{
+  return scram_server_finish (sctx, mech_data);
+}
+#endif
+
+#ifdef USE_SCRAM_SHA256
+void
+_gsasl_scram_sha256_server_finish (Gsasl_session * sctx, void *mech_data)
+{
+  return scram_server_finish (sctx, mech_data);
+}
+#endif
