@@ -1,5 +1,5 @@
 /* gsasl.h --- Header file for GNU SASL Library.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010  Simon Josefsson
+ * Copyright (C) 2002-2020 Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -21,28 +21,28 @@
  */
 
 #ifndef GSASL_H
-# define GSASL_H
+#define GSASL_H
 
-# include <stdio.h>		/* FILE */
-# include <stddef.h>		/* size_t */
-# include <unistd.h>		/* ssize_t */
+#include <stdio.h>		/* FILE */
+#include <stddef.h>		/* size_t */
+#include <unistd.h>		/* ssize_t */
 
-# ifndef GSASL_API
-#  if defined GSASL_BUILDING && defined HAVE_VISIBILITY && HAVE_VISIBILITY
-#   define GSASL_API __attribute__((__visibility__("default")))
-#  elif defined GSASL_BUILDING && defined _MSC_VER && ! defined GSASL_STATIC
-#   define GSASL_API __declspec(dllexport)
-#  elif defined _MSC_VER && ! defined GSASL_STATIC
-#   define GSASL_API __declspec(dllimport)
-#  else
-#   define GSASL_API
-#  endif
-# endif
+#ifndef GSASL_API
+#if defined GSASL_BUILDING && defined HAVE_VISIBILITY && HAVE_VISIBILITY
+#define GSASL_API __attribute__((__visibility__("default")))
+#elif defined GSASL_BUILDING && defined _MSC_VER && ! defined GSASL_STATIC
+#define GSASL_API __declspec(dllexport)
+#elif defined _MSC_VER && ! defined GSASL_STATIC
+#define GSASL_API __declspec(dllimport)
+#else
+#define GSASL_API
+#endif
+#endif
 
-# ifdef __cplusplus
+#ifdef __cplusplus
 extern "C"
 {
-# endif
+#endif
 
   /**
    * GSASL_VERSION
@@ -51,7 +51,7 @@ extern "C"
    * version number.  Used together with gsasl_check_version() to
    * verify header file and run-time library consistency.
    */
-# define GSASL_VERSION "1.5.3"
+#define GSASL_VERSION "1.9.0"
 
   /**
    * GSASL_VERSION_MAJOR
@@ -62,7 +62,7 @@ extern "C"
    *
    * Since: 1.1
    */
-# define GSASL_VERSION_MAJOR 1
+#define GSASL_VERSION_MAJOR 1
 
   /**
    * GSASL_VERSION_MINOR
@@ -73,7 +73,7 @@ extern "C"
    *
    * Since: 1.1
    */
-# define GSASL_VERSION_MINOR 5
+#define GSASL_VERSION_MINOR 9
 
   /**
    * GSASL_VERSION_PATCH
@@ -84,7 +84,7 @@ extern "C"
    *
    * Since: 1.1
    */
-# define GSASL_VERSION_PATCH 3
+#define GSASL_VERSION_PATCH 0
 
   /**
    * GSASL_VERSION_NUMBER
@@ -95,7 +95,7 @@ extern "C"
    *
    * Since: 1.1
    */
-# define GSASL_VERSION_NUMBER 0x010503
+#define GSASL_VERSION_NUMBER 0x010900
 
   /* RFC 2222: SASL mechanisms are named by strings, from 1 to 20
    * characters in length, consisting of upper-case letters, digits,
@@ -136,6 +136,11 @@ extern "C"
    * @GSASL_NO_SERVICE: Could not get required service name.
    * @GSASL_NO_HOSTNAME: Could not get required hostname.
    * @GSASL_NO_CB_TLS_UNIQUE: Could not get required tls-unique CB.
+   * @GSASL_NO_SAML20_IDP_IDENTIFIER: Could not get required SAML IdP.
+   * @GSASL_NO_SAML20_REDIRECT_URL: Could not get required SAML
+   *   redirect URL.
+   * @GSASL_NO_OPENID20_REDIRECT_URL: Could not get required OpenID
+   *   redirect URL.
    * @GSASL_GSSAPI_RELEASE_BUFFER_ERROR: GSS-API library call error.
    * @GSASL_GSSAPI_IMPORT_NAME_ERROR: GSS-API library call error.
    * @GSASL_GSSAPI_INIT_SEC_CONTEXT_ERROR: GSS-API library call error.
@@ -186,6 +191,9 @@ extern "C"
     GSASL_NO_SERVICE = 58,
     GSASL_NO_HOSTNAME = 59,
     GSASL_NO_CB_TLS_UNIQUE = 65,
+    GSASL_NO_SAML20_IDP_IDENTIFIER = 66,
+    GSASL_NO_SAML20_REDIRECT_URL = 67,
+    GSASL_NO_OPENID20_REDIRECT_URL = 68,
     /* Mechanism specific errors. */
     GSASL_GSSAPI_RELEASE_BUFFER_ERROR = 37,
     GSASL_GSSAPI_IMPORT_NAME_ERROR = 38,
@@ -206,8 +214,8 @@ extern "C"
     GSASL_GSSAPI_INQUIRE_MECH_FOR_SASLNAME_ERROR = 62,
     GSASL_GSSAPI_TEST_OID_SET_MEMBER_ERROR = 63,
     GSASL_GSSAPI_RELEASE_OID_SET_ERROR = 64
-    /* When adding new values, note that integers are not necessarily
-       assigned monotonously increasingly. */
+      /* When adding new values, note that integers are not necessarily
+         assigned monotonously increasingly. */
   } Gsasl_rc;
 
   /**
@@ -300,11 +308,21 @@ extern "C"
    * @GSASL_SCRAM_SALTED_PASSWORD: Pre-computed salted SCRAM key,
    *   to avoid re-computation and storing passwords in the clear.
    * @GSASL_CB_TLS_UNIQUE: Base64 encoded tls-unique channel binding.
+   * @GSASL_SAML20_IDP_IDENTIFIER: SAML20 user IdP URL.
+   * @GSASL_SAML20_REDIRECT_URL: SAML 2.0 URL to access in browser.
+   * @GSASL_OPENID20_REDIRECT_URL: OpenID 2.0 URL to access in browser.
+   * @GSASL_OPENID20_OUTCOME_DATA: OpenID 2.0 authentication outcome data.
+   * @GSASL_SAML20_AUTHENTICATE_IN_BROWSER: Request to perform SAML 2.0
+   *   authentication in browser.
+   * @GSASL_OPENID20_AUTHENTICATE_IN_BROWSER: Request to perform OpenID 2.0
+   *   authentication in browser.
    * @GSASL_VALIDATE_SIMPLE: Request for simple validation.
    * @GSASL_VALIDATE_EXTERNAL: Request for validation of EXTERNAL.
    * @GSASL_VALIDATE_ANONYMOUS: Request for validation of ANONYMOUS.
    * @GSASL_VALIDATE_GSSAPI: Request for validation of GSSAPI/GS2.
    * @GSASL_VALIDATE_SECURID: Reqest for validation of SecurID.
+   * @GSASL_VALIDATE_SAML20: Reqest for validation of SAML20.
+   * @GSASL_VALIDATE_OPENID20: Reqest for validation of OpenID 2.0 login.
    *
    * Callback/property types.
    */
@@ -329,12 +347,21 @@ extern "C"
     GSASL_SCRAM_SALT = 16,
     GSASL_SCRAM_SALTED_PASSWORD = 17,
     GSASL_CB_TLS_UNIQUE = 18,
+    GSASL_SAML20_IDP_IDENTIFIER = 19,
+    GSASL_SAML20_REDIRECT_URL = 20,
+    GSASL_OPENID20_REDIRECT_URL = 21,
+    GSASL_OPENID20_OUTCOME_DATA = 22,
+    /* Client callbacks. */
+    GSASL_SAML20_AUTHENTICATE_IN_BROWSER = 250,
+    GSASL_OPENID20_AUTHENTICATE_IN_BROWSER = 251,
     /* Server validation callback properties. */
     GSASL_VALIDATE_SIMPLE = 500,
     GSASL_VALIDATE_EXTERNAL = 501,
     GSASL_VALIDATE_ANONYMOUS = 502,
     GSASL_VALIDATE_GSSAPI = 503,
-    GSASL_VALIDATE_SECURID = 504
+    GSASL_VALIDATE_SECURID = 504,
+    GSASL_VALIDATE_SAML20 = 505,
+    GSASL_VALIDATE_OPENID20 = 506
   } Gsasl_property;
 
   /**
@@ -445,27 +472,27 @@ extern "C"
   extern GSASL_API int gsasl_nonce (char *data, size_t datalen);
   extern GSASL_API int gsasl_random (char *data, size_t datalen);
   extern GSASL_API int gsasl_md5 (const char *in, size_t inlen,
-				  char *out[16]);
+				  char *out[]);
   extern GSASL_API int gsasl_hmac_md5 (const char *key, size_t keylen,
 				       const char *in, size_t inlen,
-				       char *outhash[16]);
+				       char *outhash[]);
   extern GSASL_API int gsasl_sha1 (const char *in, size_t inlen,
-				   char *out[20]);
+				   char *out[]);
   extern GSASL_API int gsasl_hmac_sha1 (const char *key, size_t keylen,
 					const char *in, size_t inlen,
-					char *outhash[20]);
+					char *outhash[]);
   extern GSASL_API void gsasl_free (void *ptr);
 
   /* Get the mechanism API. */
-# include <gsasl-mech.h>
+#include <gsasl-mech.h>
 
 #ifndef GSASL_NO_OBSOLETE
   /* For compatibility with earlier versions. */
-# include <gsasl-compat.h>
+#include <gsasl-compat.h>
 #endif
 
-# ifdef __cplusplus
+#ifdef __cplusplus
 }
-# endif
+#endif
 
 #endif				/* GSASL_H */
